/**
 * This file was automatically generated by openapi-codegeneration.
 * DO NOT MODIFY IT BY HAND. Instead, modify the source OpenAPI file,
 * and run openapi-codegeneration to regenerate this file.
 */
import { AuthenticationServiceRequestValidation, BookingServiceRequestValidation, DeviceServiceRequestValidation, ExperimentServiceRequestValidation, FederationServiceRequestValidation, UpdateServiceRequestValidation, } from './requestValidation';
import { isErrorResponse, } from './types';
import fetch from 'cross-fetch';
/**
 * This error class should be used if an error occurs during validation of a request/response.
 * @category Errors
 */
export class ValidationError extends Error {
    constructor(message, errors) {
        super(message);
        this.name = 'ValidationError';
        this.errors = errors;
    }
}
/**
 * This error class should be used if an error occurs while fetching a request.
 * @category Errors
 */
export class FetchError extends Error {
    constructor(message) {
        super(message);
        this.name = 'FetchError';
    }
}
/**
 * This error class should be used if the response of the server has a status
 * greater than or equal to 400. This error should contain the validated response.
 * @category Errors
 */
export class UnsuccessfulRequestError extends Error {
    constructor(message, response) {
        super(message);
        this.response = response;
        this.name = 'UnsuccessfulRequestError';
    }
}
/**
 * This error class should be used if the validation of an url fails.
 * @category Errors
 */
export class InvalidUrlError extends Error {
    constructor(message) {
        super(message);
        this.name = 'InvalidUrlError';
    }
}
/**
 * This function attempts to parse a Response as ResponseData.
 */
async function parseResponse(response) {
    let text = null;
    try {
        text = await response.text();
        const json = JSON.parse(text);
        return { status: response.status, body: json };
    }
    catch {
        return { status: response.status, body: text };
    }
}
/**
 * This function checks if a string is a valid http url.
 * @param string The string to be checked.
 * @returns True if the string is a valid http url.
 */
function isValidHttpUrl(string) {
    let url;
    try {
        url = new URL(string);
    }
    catch (_) {
        return false;
    }
    return url.protocol === 'http:' || url.protocol === 'https:';
}
/**
 * This function takes an url and a pattern for the requested endpoint and parses the contained path parameters.
 * @param url The url to be parsed.
 * @param endpoint Pattern of the requested endpoint.
 * @throws {InvalidUrlError} Thrown if the provided url does not end with the provided endpoint.
 * @returns An array containing all parsed path parameters in order of appearance.
 * @example
 * // returns ["username", "role_name"]
 * parsePathParameters("https://api.example.com/users/username/roles/role_name", "/users/{}/roles/{}")
 */
function parsePathParameters(url, endpoint) {
    const parameterRegex = '([a-zA-Z0-9-:]+)';
    const regex = new RegExp(endpoint.replaceAll('{}', parameterRegex) + '(?:.(?!\\\\))?$');
    const matches = url.match(regex);
    if (!matches)
        throw new InvalidUrlError('Url does not end with the provided endpoint');
    return matches.slice(1);
}
/**
 * This function validates a given http url and parses its path parameters.
 * @param url The url to be validated.
 * @param endpoint Pattern of the requested endpoint.
 * @throws {InvalidUrlError} Thrown if the validation of the provided url fails.
 * @returns An array containing all parsed path parameters in order of appearance (see {@link parsePathParameters}).
 * @example
 * // returns ["username", "role_name"]
 * validateUrl("https://api.example.com/users/username/roles/role_name", "/users/{}/roles/{}")
 */
function validateUrl(url, endpoint) {
    if (!isValidHttpUrl(url))
        throw new InvalidUrlError('Provided url is not a valid http url');
    return parsePathParameters(url, endpoint);
}
/**
 * This function appends a given endpoint to an url.
 * @param url The url to append the endpoint to.
 * @param endpoint The endpoint to be appended.
 * @example
 * // returns "https://api.example.com/devices/device_id/token"
 * appendToUrl("https://api.example.com/devices/device_id", "/token")
 * @example
 * // returns "https://api.example.com/devices/device_id/token"
 * appendToUrl("https://api.example.com/devices/device_id/", "/token")
 * @returns The url with the appended endpoint.
 */
function appendToUrl(url, endpoint) {
    return url.endsWith('/')
        ? `${url.slice(0, -1)}${endpoint}`
        : `${url}${endpoint}`;
}
/**
 * Client for the use of the Crosslab API
 * @category Client
 */
export class APIClient {
    /**
     * @category Constructors
     */
    constructor(url, accessToken) {
        this.fetch = async (url, info) => {
            let raw_response;
            try {
                if (url.startsWith(this._url)) {
                    raw_response = await fetch(url, info);
                }
                else {
                    raw_response = await fetch(appendToUrl(this._url, '/proxy?' +
                        new URLSearchParams([['URL', url]]).toString()), info);
                }
            }
            catch (error) {
                if (error instanceof Error) {
                    throw new FetchError(error.message);
                }
                else if (typeof error === 'string') {
                    throw new FetchError(error);
                }
                else {
                    throw new FetchError('Something went wrong while trying to fetch the request');
                }
            }
            return await parseResponse(raw_response);
        };
        this._url = url.endsWith('/') ? url.slice(0, -1) : url;
        this._accessToken = accessToken ?? '';
    }
    /**
     * @category Accessors
     */
    get url() {
        return this._url;
    }
    /**
     * @category Accessors
     */
    set url(url) {
        this._url = url.endsWith('/') ? url.slice(0, -1) : url;
    }
    /**
     * @category Accessors
     */
    set accessToken(accessToken) {
        this._accessToken = accessToken;
    }
    /**
     * @category Accessors
     */
    get accessToken() {
        return this._accessToken;
    }
    /**
     * Login user
     *
     * @param username
     * Username of the user.
     * @param password
     * Password of the user.
     * @param options.method
     * Authentication method.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Return the access token.
     *
     * @category Authentication Service Methods
     */
    async login(username, password, options) {
        const url = appendToUrl(this._url, '/login');
        const body = {
            username: username,
            password: password,
            method: options?.method,
        };
        if (!AuthenticationServiceRequestValidation.validateLoginInput(body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateLoginInput.errors);
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [['Content-Type', 'application/json']],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateLoginOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateLoginOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        this._accessToken = response.body;
    }
    /**
     * Logout user
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Successfully logged out the user.
     *
     * @category Authentication Service Methods
     */
    async logout() {
        const url = appendToUrl(this._url, '/logout');
        const body = {
            token: this._accessToken,
        };
        if (!AuthenticationServiceRequestValidation.validateLogoutInput(body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateLogoutInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateLogoutOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateLogoutOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        this._accessToken = '';
    }
    /**
     * Create a device authentication token
     *
     * @param device_url
     * Url of the device.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The created device authentication token.
     *
     * @category Authentication Service Methods
     */
    async createDeviceAuthenticationToken(device_url) {
        const url = appendToUrl(this._url, '/device_authentication_token');
        const parameters = {
            device_url: device_url,
        };
        const query = [];
        if (parameters['device_url'])
            query.push(['device_url', parameters['device_url'].toString()]);
        if (!AuthenticationServiceRequestValidation.validateCreateDeviceAuthenticationTokenInput(parameters))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateCreateDeviceAuthenticationTokenInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateCreateDeviceAuthenticationTokenOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateCreateDeviceAuthenticationTokenOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Get all users
     *
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the users.
     *
     * @category Authentication Service Methods
     */
    async listUsers(options) {
        const url = appendToUrl(options?.url ?? this._url, '/users');
        if (!AuthenticationServiceRequestValidation.validateListUsersInput())
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateListUsersInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateListUsersOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateListUsersOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create a user
     *
     * @param user
     * User to be created.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the created user.
     *
     * @category Authentication Service Methods
     */
    async createUser(user, options) {
        const url = appendToUrl(options?.url ?? this._url, '/users');
        const body = user;
        if (!AuthenticationServiceRequestValidation.validateCreateUserInput(body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateCreateUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateCreateUserOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateCreateUserOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Get a user
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON respresentation of the user.
     *
     * @category Authentication Service Methods
     */
    async getUser(url) {
        const urlSuffix = '/users/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [user_id] = validateUrl(url, '/users/{}');
        const parameters = {
            user_id: user_id,
        };
        if (!AuthenticationServiceRequestValidation.validateGetUserInput(parameters))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateGetUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateGetUserOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateGetUserOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update a user
     *
     * @param url
     * Url of the resource to be accessed.
     * @param userUpdate
     * Updated user
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the updated user.
     *
     * @category Authentication Service Methods
     */
    async updateUser(url, userUpdate) {
        const urlSuffix = '/users/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [user_id] = validateUrl(url, '/users/{}');
        const body = userUpdate;
        const parameters = {
            user_id: user_id,
        };
        if (!AuthenticationServiceRequestValidation.validateUpdateUserInput(parameters, body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateUpdateUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateUpdateUserOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateUpdateUserOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete a user
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * User deleted successfully
     *
     * @category Authentication Service Methods
     */
    async deleteUser(url) {
        const urlSuffix = '/users/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [user_id] = validateUrl(url, '/users/{}');
        const parameters = {
            user_id: user_id,
        };
        if (!AuthenticationServiceRequestValidation.validateDeleteUserInput(parameters))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateDeleteUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateDeleteUserOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateDeleteUserOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Get roles of user
     *
     * @param user_id
     * ID of a user.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Roles successfully retrieved.
     *
     * @category Authentication Service Methods
     */
    async getRolesOfUser(user_id, options) {
        const url = appendToUrl(options?.url ?? this._url, `/users/${user_id}/roles`);
        const parameters = {
            user_id: user_id,
        };
        if (!AuthenticationServiceRequestValidation.validateGetRolesOfUserInput(parameters))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateGetRolesOfUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateGetRolesOfUserOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateGetRolesOfUserOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Add roles to user
     *
     * @param body
     * Roles to be added to the user.
     * @param user_id
     * ID of a user.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Roles added successfully.
     *
     * @category Authentication Service Methods
     */
    async addRolesToUser(body, user_id, options) {
        const url = appendToUrl(options?.url ?? this._url, `/users/${user_id}/roles`);
        const parameters = {
            user_id: user_id,
        };
        if (!AuthenticationServiceRequestValidation.validateAddRolesToUserInput(parameters, body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateAddRolesToUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateAddRolesToUserOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateAddRolesToUserOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Remove roles from user
     *
     * @param body
     * Roles to be removed from the user.
     * @param user_id
     * ID of a user.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Roles removed successfully.
     *
     * @category Authentication Service Methods
     */
    async removeRolesFromUser(body, user_id, options) {
        const url = appendToUrl(options?.url ?? this._url, `/users/${user_id}/roles`);
        const parameters = {
            user_id: user_id,
        };
        if (!AuthenticationServiceRequestValidation.validateRemoveRolesFromUserInput(parameters, body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateRemoveRolesFromUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateRemoveRolesFromUserOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateRemoveRolesFromUserOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Get all roles
     *
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the roles.
     *
     * @category Authentication Service Methods
     */
    async listRoles(options) {
        const url = appendToUrl(options?.url ?? this._url, '/roles');
        if (!AuthenticationServiceRequestValidation.validateListRolesInput())
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateListRolesInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateListRolesOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateListRolesOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create a role
     *
     * @param role
     * Role to be created.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the created role.
     *
     * @category Authentication Service Methods
     */
    async createRole(role, options) {
        const url = appendToUrl(options?.url ?? this._url, '/roles');
        const body = role;
        if (!AuthenticationServiceRequestValidation.validateCreateRoleInput(body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateCreateRoleInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateCreateRoleOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateCreateRoleOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Get a role
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON respresentation of the role.
     *
     * @category Authentication Service Methods
     */
    async getRole(url) {
        const urlSuffix = '/roles/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [role_id] = validateUrl(url, '/roles/{}');
        const parameters = {
            role_id: role_id,
        };
        if (!AuthenticationServiceRequestValidation.validateGetRoleInput(parameters))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateGetRoleInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateGetRoleOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateGetRoleOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update a role
     *
     * @param url
     * Url of the resource to be accessed.
     * @param role
     * Updated role
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the updated role.
     *
     * @category Authentication Service Methods
     */
    async updateRole(url, role) {
        const urlSuffix = '/roles/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [role_id] = validateUrl(url, '/roles/{}');
        const body = role;
        const parameters = {
            role_id: role_id,
        };
        if (!AuthenticationServiceRequestValidation.validateUpdateRoleInput(parameters, body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateUpdateRoleInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateUpdateRoleOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateUpdateRoleOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete a role
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Role deleted successfully
     *
     * @category Authentication Service Methods
     */
    async deleteRole(url) {
        const urlSuffix = '/roles/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [role_id] = validateUrl(url, '/roles/{}');
        const parameters = {
            role_id: role_id,
        };
        if (!AuthenticationServiceRequestValidation.validateDeleteRoleInput(parameters))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateDeleteRoleInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateDeleteRoleOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateDeleteRoleOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Get users with role
     *
     * @param role_id
     * ID of a role.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Users successfully retrieved.
     *
     * @category Authentication Service Methods
     */
    async getUsersWithRole(role_id, options) {
        const url = appendToUrl(options?.url ?? this._url, `/roles/${role_id}/users`);
        const parameters = {
            role_id: role_id,
        };
        if (!AuthenticationServiceRequestValidation.validateGetUsersWithRoleInput(parameters))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateGetUsersWithRoleInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateGetUsersWithRoleOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateGetUsersWithRoleOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Add users to role
     *
     * @param body
     * Users to be added to the role.
     * @param role_id
     * ID of a role.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Users added successfully.
     *
     * @category Authentication Service Methods
     */
    async addUsersToRole(body, role_id, options) {
        const url = appendToUrl(options?.url ?? this._url, `/roles/${role_id}/users`);
        const parameters = {
            role_id: role_id,
        };
        if (!AuthenticationServiceRequestValidation.validateAddUsersToRoleInput(parameters, body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateAddUsersToRoleInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateAddUsersToRoleOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateAddUsersToRoleOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Remove users from role
     *
     * @param body
     * Users to be removed from the role.
     * @param role_id
     * ID of a role.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Users removed successfully.
     *
     * @category Authentication Service Methods
     */
    async removeUsersFromRole(body, role_id, options) {
        const url = appendToUrl(options?.url ?? this._url, `/roles/${role_id}/users`);
        const parameters = {
            role_id: role_id,
        };
        if (!AuthenticationServiceRequestValidation.validateRemoveUsersFromRoleInput(parameters, body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateRemoveUsersFromRoleInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateRemoveUsersFromRoleOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateRemoveUsersFromRoleOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Get identity
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the corresponding user.
     *
     * @category Authentication Service Methods
     */
    async getIdentity() {
        const url = appendToUrl(this._url, '/identity');
        if (!AuthenticationServiceRequestValidation.validateGetIdentityInput())
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateGetIdentityInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!AuthenticationServiceRequestValidation.validateGetIdentityOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateGetIdentityOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update identity
     *
     * @param userUpdate
     * Updated identity.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the updated user.
     *
     * @category Authentication Service Methods
     */
    async updateIdentity(userUpdate) {
        const url = appendToUrl(this._url, '/identity');
        const body = userUpdate;
        if (!AuthenticationServiceRequestValidation.validateUpdateIdentityInput(body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateUpdateIdentityInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateUpdateIdentityOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateUpdateIdentityOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Register user
     *
     * @param body
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * User successfully registered.
     *
     * @category Authentication Service Methods
     */
    async register(body) {
        const url = appendToUrl(this._url, '/register');
        if (!AuthenticationServiceRequestValidation.validateRegisterInput(body))
            throw new ValidationError('Request validation failed!', AuthenticationServiceRequestValidation.validateRegisterInput.errors);
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [['Content-Type', 'application/json']],
            body: JSON.stringify(body),
        });
        if (!AuthenticationServiceRequestValidation.validateRegisterOutput(response))
            throw new ValidationError('Response validation failed!', AuthenticationServiceRequestValidation.validateRegisterOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Returns the free / booked times for given experiment.
     *
     * @param scheduleRequest
     * Requested experiment and timeframe.
     * @param options.url
     * Url of the Booking Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Timetable of free/booked time. If 'Combined' is set to true, an array with only one entry is returned containing the combined timetable.
     *
     * @category Booking Service Methods
     */
    async getSchedule(scheduleRequest, options) {
        const url = appendToUrl(options?.url ?? this._url, '/schedule');
        const body = scheduleRequest;
        if (!BookingServiceRequestValidation.validateGetScheduleInput(body))
            throw new ValidationError('Request validation failed!', BookingServiceRequestValidation.validateGetScheduleInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!BookingServiceRequestValidation.validateGetScheduleOutput(response))
            throw new ValidationError('Response validation failed!', BookingServiceRequestValidation.validateGetScheduleOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Books an experiment.
     *
     * @param bookingRequest
     * Experiment, timeframe and type of booking.
     * @param options.url
     * Url of the Booking Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The booking was accepted. The booking status will display whether the booking of all devices was successful.
     *
     * @category Booking Service Methods
     */
    async bookExperiment(bookingRequest, options) {
        const url = appendToUrl(options?.url ?? this._url, '/booking');
        const body = bookingRequest;
        if (!BookingServiceRequestValidation.validateBookExperimentInput(body))
            throw new ValidationError('Request validation failed!', BookingServiceRequestValidation.validateBookExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!BookingServiceRequestValidation.validateBookExperimentOutput(response))
            throw new ValidationError('Response validation failed!', BookingServiceRequestValidation.validateBookExperimentOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Allows the addition of devices to a booking (removing of devices is not supportet) or the registration of callbacks.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param devicesOrCallback
     * Devices or callback to be added to the booking.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The booking change was accepted. The booking status will display whether the booking of all devices was successful.
     *
     * @category Booking Service Methods
     */
    async updateBooking(url, devicesOrCallback) {
        const urlSuffix = '/booking/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [ID] = validateUrl(url, '/booking/{}');
        const body = devicesOrCallback;
        const parameters = {
            ID: ID,
        };
        if (!BookingServiceRequestValidation.validateUpdateBookingInput(parameters, body))
            throw new ValidationError('Request validation failed!', BookingServiceRequestValidation.validateUpdateBookingInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!BookingServiceRequestValidation.validateUpdateBookingOutput(response))
            throw new ValidationError('Response validation failed!', BookingServiceRequestValidation.validateUpdateBookingOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Cancels a booking, as long as the booking was originally done by you.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The booking was cancelled. All associated devices were released.
     *
     * @category Booking Service Methods
     */
    async cancelBooking(url) {
        const urlSuffix = '/booking/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [ID] = validateUrl(url, '/booking/{}');
        const parameters = {
            ID: ID,
        };
        if (!BookingServiceRequestValidation.validateCancelBookingInput(parameters))
            throw new ValidationError('Request validation failed!', BookingServiceRequestValidation.validateCancelBookingInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!BookingServiceRequestValidation.validateCancelBookingOutput(response))
            throw new ValidationError('Response validation failed!', BookingServiceRequestValidation.validateCancelBookingOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Returns whether a list of devices is currently booked for a user
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Returns the current booking.
     *
     * @category Booking Service Methods
     */
    async getBooking(url) {
        const urlSuffix = '/booking/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [ID] = validateUrl(url, '/booking/{}');
        const parameters = {
            ID: ID,
        };
        if (!BookingServiceRequestValidation.validateGetBookingInput(parameters))
            throw new ValidationError('Request validation failed!', BookingServiceRequestValidation.validateGetBookingInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!BookingServiceRequestValidation.validateGetBookingOutput(response))
            throw new ValidationError('Response validation failed!', BookingServiceRequestValidation.validateGetBookingOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Allows selected persons (like lab manager) to remove a user booking. To avoid mistakes, this is a different path than normal delete.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The booking was cancelled. All associated devices were released.
     *
     * @category Booking Service Methods
     */
    async deleteBooking(url) {
        const urlSuffix = '/booking/{}/destroy'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [ID] = validateUrl(url, '/booking/{}/destroy');
        const parameters = {
            ID: ID,
        };
        if (!BookingServiceRequestValidation.validateDeleteBookingInput(parameters))
            throw new ValidationError('Request validation failed!', BookingServiceRequestValidation.validateDeleteBookingInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!BookingServiceRequestValidation.validateDeleteBookingOutput(response))
            throw new ValidationError('Response validation failed!', BookingServiceRequestValidation.validateDeleteBookingOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Locks the current booking so the devices can be used. This sets the status to "active" This means that the booking can not be cancelled or (currently not implemented) the end time can not be set to a prior time. If called multiple times, the booking will be locked only once.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * All devices were booked by user.
     *
     * @category Booking Service Methods
     */
    async lockBooking(url) {
        const urlSuffix = '/booking/{}/lock'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [ID] = validateUrl(url, '/booking/{}/lock');
        const parameters = {
            ID: ID,
        };
        if (!BookingServiceRequestValidation.validateLockBookingInput(parameters))
            throw new ValidationError('Request validation failed!', BookingServiceRequestValidation.validateLockBookingInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'PUT',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!BookingServiceRequestValidation.validateLockBookingOutput(response))
            throw new ValidationError('Response validation failed!', BookingServiceRequestValidation.validateLockBookingOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Unlocks all devices belonging to a booking, status will be set to 'booked'.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Unlocking was successful.
     *
     * @category Booking Service Methods
     */
    async unlockBooking(url) {
        const urlSuffix = '/booking/{}/lock'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [ID] = validateUrl(url, '/booking/{}/lock');
        const parameters = {
            ID: ID,
        };
        if (!BookingServiceRequestValidation.validateUnlockBookingInput(parameters))
            throw new ValidationError('Request validation failed!', BookingServiceRequestValidation.validateUnlockBookingInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!BookingServiceRequestValidation.validateUnlockBookingOutput(response))
            throw new ValidationError('Response validation failed!', BookingServiceRequestValidation.validateUnlockBookingOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * List devices
     *
     * @param options.url
     * Url of the Device Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all devices.
     *
     * @category Device Service Methods
     */
    async listDevices(options) {
        const url = appendToUrl(options?.url ?? this._url, '/devices');
        if (!DeviceServiceRequestValidation.validateListDevicesInput())
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateListDevicesInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!DeviceServiceRequestValidation.validateListDevicesOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateListDevicesOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create a new device
     *
     * @param device
     * The device to be created.
     * @param options.changedUrl
     * **An URL that will be called once the device changes.**
     *
     * Once the device was given a changedUrl parameter the contained URL will be called every time a device is changed
     * as long as every callback resolves with a successful status code.
     *
     * If the callback fails the url MIGHT not be called in the future.
     *
     * There can be multiple callbacks registered with the same device.
     * @param options.url
     * Url of the Device Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The device was created. A JSON representation of the new device is returned.
     *
     * @category Device Service Methods
     */
    async createDevice(device, options) {
        const url = appendToUrl(options?.url ?? this._url, '/devices');
        const body = device;
        const parameters = {
            changedUrl: options?.changedUrl,
        };
        const query = [];
        if (parameters['changedUrl'])
            query.push(['changedUrl', parameters['changedUrl'].toString()]);
        if (!DeviceServiceRequestValidation.validateCreateDeviceInput(parameters, body))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateCreateDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!DeviceServiceRequestValidation.validateCreateDeviceOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateCreateDeviceOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * View a registered device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param options.flat_group
     * If true the returned device group will only contain concrete devices. I.e. any subgroups will be flattend.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the device.
     *
     * @category Device Service Methods
     */
    async getDevice(url, options) {
        const urlSuffix = '/devices/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/devices/{}');
        const parameters = {
            device_id: device_id,
            flat_group: options?.flat_group,
        };
        const query = [];
        if (parameters['flat_group'])
            query.push(['flat_group', parameters['flat_group'].toString()]);
        if (!DeviceServiceRequestValidation.validateGetDeviceInput(parameters))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateGetDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!DeviceServiceRequestValidation.validateGetDeviceOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateGetDeviceOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update an existing device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param deviceUpdate
     * Updated device.
     * @param options.changedUrl
     * **An URL that will be called once the device changes.**
     *
     * Once the device was given a changedUrl parameter the contained URL will be called every time a device is changed
     * as long as every callback resolves with a successful status code.
     *
     * If the callback fails the url MIGHT not be called in the future.
     *
     * There can be multiple callbacks registered with the same device.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the changed device.
     *
     * @category Device Service Methods
     */
    async updateDevice(url, deviceUpdate, options) {
        const urlSuffix = '/devices/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/devices/{}');
        const body = deviceUpdate;
        const parameters = {
            device_id: device_id,
            changedUrl: options?.changedUrl,
        };
        const query = [];
        if (parameters['changedUrl'])
            query.push(['changedUrl', parameters['changedUrl'].toString()]);
        if (!DeviceServiceRequestValidation.validateUpdateDeviceInput(parameters, body))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateUpdateDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!DeviceServiceRequestValidation.validateUpdateDeviceOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateUpdateDeviceOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete a registered device
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The device was deleted.
     *
     * @category Device Service Methods
     */
    async deleteDevice(url) {
        const urlSuffix = '/devices/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/devices/{}');
        const parameters = {
            device_id: device_id,
        };
        if (!DeviceServiceRequestValidation.validateDeleteDeviceInput(parameters))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateDeleteDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!DeviceServiceRequestValidation.validateDeleteDeviceOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateDeleteDeviceOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Instantiate a cloud instantiable device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param options.changedUrl
     * **An URL that will be called once the device changes.**
     *
     * Once the device was given a changedUrl parameter the contained URL will be called every time a device is changed
     * as long as every callback resolves with a successful status code.
     *
     * If the callback fails the url MIGHT not be called in the future.
     *
     * There can be multiple callbacks registered with the same device.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the created instance.
     *
     * @category Device Service Methods
     */
    async instantiateDevice(url, options) {
        const urlSuffix = '/devices/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/devices/{}');
        const parameters = {
            device_id: device_id,
            changedUrl: options?.changedUrl,
        };
        const query = [];
        if (parameters['changedUrl'])
            query.push(['changedUrl', parameters['changedUrl'].toString()]);
        if (!DeviceServiceRequestValidation.validateInstantiateDeviceInput(parameters))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateInstantiateDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!DeviceServiceRequestValidation.validateInstantiateDeviceOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateInstantiateDeviceOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update the device availability
     *
     * @param url
     * Url of the resource to be accessed.
     * @param availabilityRules
     * The availability rules to be applied.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the changed availability.
     *
     * @category Device Service Methods
     */
    async addAvailabilityRules(url, availabilityRules) {
        const urlSuffix = '/devices/{}/availability'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/devices/{}/availability');
        const body = availabilityRules;
        const parameters = {
            device_id: device_id,
        };
        if (!DeviceServiceRequestValidation.validateAddAvailabilityRulesInput(parameters, body))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateAddAvailabilityRulesInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!DeviceServiceRequestValidation.validateAddAvailabilityRulesOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateAddAvailabilityRulesOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create new websocket token for device
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The newly generated websocket token for the device.
     *
     * @category Device Service Methods
     */
    async createWebsocketToken(url) {
        const urlSuffix = '/devices/{}/websocket'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/devices/{}/websocket');
        const parameters = {
            device_id: device_id,
        };
        if (!DeviceServiceRequestValidation.validateCreateWebsocketTokenInput(parameters))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateCreateWebsocketTokenInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!DeviceServiceRequestValidation.validateCreateWebsocketTokenOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateCreateWebsocketTokenOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Send signaling message to device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param sigMessage
     * The signaling message to be sent.
     * @param peerconnection_url
     * URL of the peerconnection
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The signaling message was delivered successfully.
     *
     * @category Device Service Methods
     */
    async sendSignalingMessage(url, sigMessage, peerconnection_url) {
        const urlSuffix = '/devices/{}/signaling'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/devices/{}/signaling');
        const body = sigMessage;
        const parameters = {
            device_id: device_id,
            peerconnection_url: peerconnection_url,
        };
        const query = [];
        if (parameters['peerconnection_url'])
            query.push([
                'peerconnection_url',
                parameters['peerconnection_url'].toString(),
            ]);
        if (!DeviceServiceRequestValidation.validateSendSignalingMessageInput(parameters, body))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateSendSignalingMessageInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!DeviceServiceRequestValidation.validateSendSignalingMessageOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateSendSignalingMessageOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * List Peer Connection
     *
     * @param options.url
     * Url of the Device Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all peerconnections.
     *
     * @category Device Service Methods
     */
    async listPeerconnections(options) {
        const url = appendToUrl(options?.url ?? this._url, '/peerconnections');
        if (!DeviceServiceRequestValidation.validateListPeerconnectionsInput())
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateListPeerconnectionsInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!DeviceServiceRequestValidation.validateListPeerconnectionsOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateListPeerconnectionsOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create a new Peer Connection
     *
     * @param peerconnection
     * The peerconnection to be created.
     * @param options.closedUrl
     * An URL that will be called once the peer connection is closed.
     * @param options.statusChangedUrl
     * An URL that will be called if the status of the peerconnection changes.
     * @param options.url
     * Url of the Device Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 201: The peerconnection was created. A JSON representation of the new peer connection is returned.
     * 202: The device service will attempt to create the peerconnection as soon as possible.
     *
     * @category Device Service Methods
     */
    async createPeerconnection(peerconnection, options) {
        const url = appendToUrl(options?.url ?? this._url, '/peerconnections');
        const body = peerconnection;
        const parameters = {
            closedUrl: options?.closedUrl,
            statusChangedUrl: options?.statusChangedUrl,
        };
        const query = [];
        if (parameters['closedUrl'])
            query.push(['closedUrl', parameters['closedUrl'].toString()]);
        if (parameters['statusChangedUrl'])
            query.push([
                'statusChangedUrl',
                parameters['statusChangedUrl'].toString(),
            ]);
        if (!DeviceServiceRequestValidation.validateCreatePeerconnectionInput(parameters, body))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateCreatePeerconnectionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!DeviceServiceRequestValidation.validateCreatePeerconnectionOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateCreatePeerconnectionOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * View a peer connection
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the device peer connection.
     *
     * @category Device Service Methods
     */
    async getPeerconnection(url) {
        const urlSuffix = '/peerconnections/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [peerconnection_id] = validateUrl(url, '/peerconnections/{}');
        const parameters = {
            peerconnection_id: peerconnection_id,
        };
        if (!DeviceServiceRequestValidation.validateGetPeerconnectionInput(parameters))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateGetPeerconnectionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!DeviceServiceRequestValidation.validateGetPeerconnectionOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateGetPeerconnectionOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete a peer connection
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The peer connection was deleted successfully.
     *
     * @category Device Service Methods
     */
    async deletePeerconnection(url) {
        const urlSuffix = '/peerconnections/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [peerconnection_id] = validateUrl(url, '/peerconnections/{}');
        const parameters = {
            peerconnection_id: peerconnection_id,
        };
        if (!DeviceServiceRequestValidation.validateDeletePeerconnectionInput(parameters))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validateDeletePeerconnectionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!DeviceServiceRequestValidation.validateDeletePeerconnectionOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validateDeletePeerconnectionOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Sets the peerconnection status of a single device.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param body
     * The JSON Representation of the device peer connection.
     * @param device_url
     * URL of the device
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * OK.
     *
     * @category Device Service Methods
     */
    async patchPeerconnectionDeviceStatus(url, body, device_url) {
        const urlSuffix = '/peerconnections/{}/device_status'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [peerconnection_id] = validateUrl(url, '/peerconnections/{}/device_status');
        const parameters = {
            peerconnection_id: peerconnection_id,
            device_url: device_url,
        };
        const query = [];
        if (parameters['device_url'])
            query.push(['device_url', parameters['device_url'].toString()]);
        if (!DeviceServiceRequestValidation.validatePatchPeerconnectionDeviceStatusInput(parameters, body))
            throw new ValidationError('Request validation failed!', DeviceServiceRequestValidation.validatePatchPeerconnectionDeviceStatusInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!DeviceServiceRequestValidation.validatePatchPeerconnectionDeviceStatusOutput(response))
            throw new ValidationError('Response validation failed!', DeviceServiceRequestValidation.validatePatchPeerconnectionDeviceStatusOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * List experiments
     *
     * @param options.url
     * Url of the Experiment Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all experiments.
     *
     * @category Experiment Service Methods
     */
    async listExperiments(options) {
        const url = appendToUrl(options?.url ?? this._url, '/experiments');
        if (!ExperimentServiceRequestValidation.validateListExperimentsInput())
            throw new ValidationError('Request validation failed!', ExperimentServiceRequestValidation.validateListExperimentsInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!ExperimentServiceRequestValidation.validateListExperimentsOutput(response))
            throw new ValidationError('Response validation failed!', ExperimentServiceRequestValidation.validateListExperimentsOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create a new experiment
     *
     * @param experiment
     * The experiment to be created.
     * @param options.url
     * Url of the Experiment Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 201: The experiment was created. A JSON representation of the new experiment is returned.
     * 202: The experiment was created and will be set to running eventually. A JSON representation of the new experiment is returned.
     *
     * @category Experiment Service Methods
     */
    async createExperiment(experiment, options) {
        const url = appendToUrl(options?.url ?? this._url, '/experiments');
        const body = experiment;
        if (!ExperimentServiceRequestValidation.validateCreateExperimentInput(body))
            throw new ValidationError('Request validation failed!', ExperimentServiceRequestValidation.validateCreateExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!ExperimentServiceRequestValidation.validateCreateExperimentOutput(response))
            throw new ValidationError('Response validation failed!', ExperimentServiceRequestValidation.validateCreateExperimentOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * View an experiment.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the experiment
     *
     * @category Experiment Service Methods
     */
    async getExperiment(url) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id] = validateUrl(url, '/experiments/{}');
        const parameters = {
            experiment_id: experiment_id,
        };
        if (!ExperimentServiceRequestValidation.validateGetExperimentInput(parameters))
            throw new ValidationError('Request validation failed!', ExperimentServiceRequestValidation.validateGetExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!ExperimentServiceRequestValidation.validateGetExperimentOutput(response))
            throw new ValidationError('Response validation failed!', ExperimentServiceRequestValidation.validateGetExperimentOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update an existing experiment.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param experiment
     * Update the experiment
     * @param options.changedURL
     * An URL that will be called when the experiment status changes.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 200: The JSON Representation of the changed experiment
     * 202: The JSON Representation of the changed experiment, that will be set to running eventually
     *
     * @category Experiment Service Methods
     */
    async updateExperiment(url, experiment, options) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id] = validateUrl(url, '/experiments/{}');
        const body = experiment;
        const parameters = {
            experiment_id: experiment_id,
            changedURL: options?.changedURL,
        };
        const query = [];
        if (parameters['changedURL'])
            query.push(['changedURL', parameters['changedURL'].toString()]);
        if (!ExperimentServiceRequestValidation.validateUpdateExperimentInput(parameters, body))
            throw new ValidationError('Request validation failed!', ExperimentServiceRequestValidation.validateUpdateExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!ExperimentServiceRequestValidation.validateUpdateExperimentOutput(response))
            throw new ValidationError('Response validation failed!', ExperimentServiceRequestValidation.validateUpdateExperimentOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete an experiment
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The experiment was deleted.
     *
     * @category Experiment Service Methods
     */
    async deleteExperiment(url) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id] = validateUrl(url, '/experiments/{}');
        const parameters = {
            experiment_id: experiment_id,
        };
        if (!ExperimentServiceRequestValidation.validateDeleteExperimentInput(parameters))
            throw new ValidationError('Request validation failed!', ExperimentServiceRequestValidation.validateDeleteExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!ExperimentServiceRequestValidation.validateDeleteExperimentOutput(response))
            throw new ValidationError('Response validation failed!', ExperimentServiceRequestValidation.validateDeleteExperimentOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * List institutions
     *
     * @param options.url
     * Url of the Federation Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all institutions.
     *
     * @category Federation Service Methods
     */
    async listInstitutions(options) {
        const url = appendToUrl(options?.url ?? this._url, '/institutions');
        if (!FederationServiceRequestValidation.validateListInstitutionsInput())
            throw new ValidationError('Request validation failed!', FederationServiceRequestValidation.validateListInstitutionsInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!FederationServiceRequestValidation.validateListInstitutionsOutput(response))
            throw new ValidationError('Response validation failed!', FederationServiceRequestValidation.validateListInstitutionsOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create a new institution
     *
     * @param institution
     * The institution to be created.
     * @param options.url
     * Url of the Federation Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The institution was created. A JSON representation of the new institution is returned.
     *
     * @category Federation Service Methods
     */
    async createInstitution(institution, options) {
        const url = appendToUrl(options?.url ?? this._url, '/institutions');
        const body = institution;
        if (!FederationServiceRequestValidation.validateCreateInstitutionInput(body))
            throw new ValidationError('Request validation failed!', FederationServiceRequestValidation.validateCreateInstitutionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!FederationServiceRequestValidation.validateCreateInstitutionOutput(response))
            throw new ValidationError('Response validation failed!', FederationServiceRequestValidation.validateCreateInstitutionOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * View an institution.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the institution
     *
     * @category Federation Service Methods
     */
    async getInstitution(url) {
        const urlSuffix = '/institutions/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [institution_id] = validateUrl(url, '/institutions/{}');
        const parameters = {
            institution_id: institution_id,
        };
        if (!FederationServiceRequestValidation.validateGetInstitutionInput(parameters))
            throw new ValidationError('Request validation failed!', FederationServiceRequestValidation.validateGetInstitutionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!FederationServiceRequestValidation.validateGetInstitutionOutput(response))
            throw new ValidationError('Response validation failed!', FederationServiceRequestValidation.validateGetInstitutionOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Update an institution.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param institution
     * Update the institution
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the changed institution
     *
     * @category Federation Service Methods
     */
    async updateInstitution(url, institution) {
        const urlSuffix = '/institutions/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [institution_id] = validateUrl(url, '/institutions/{}');
        const body = institution;
        const parameters = {
            institution_id: institution_id,
        };
        if (!FederationServiceRequestValidation.validateUpdateInstitutionInput(parameters, body))
            throw new ValidationError('Request validation failed!', FederationServiceRequestValidation.validateUpdateInstitutionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!FederationServiceRequestValidation.validateUpdateInstitutionOutput(response))
            throw new ValidationError('Response validation failed!', FederationServiceRequestValidation.validateUpdateInstitutionOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete an institution
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The institution was delete.
     *
     * @category Federation Service Methods
     */
    async deleteInstitution(url) {
        const urlSuffix = '/institutions/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [institution_id] = validateUrl(url, '/institutions/{}');
        const parameters = {
            institution_id: institution_id,
        };
        if (!FederationServiceRequestValidation.validateDeleteInstitutionInput(parameters))
            throw new ValidationError('Request validation failed!', FederationServiceRequestValidation.validateDeleteInstitutionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!FederationServiceRequestValidation.validateDeleteInstitutionOutput(response))
            throw new ValidationError('Response validation failed!', FederationServiceRequestValidation.validateDeleteInstitutionOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Get update information for all devices
     *
     * @param options.url
     * Url of the Update Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * JSON representation of all update information
     *
     * @category Update Service Methods
     */
    async listUpdates(options) {
        const url = appendToUrl(options?.url ?? this._url, '/updates');
        if (!UpdateServiceRequestValidation.validateListUpdatesInput())
            throw new ValidationError('Request validation failed!', UpdateServiceRequestValidation.validateListUpdatesInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!UpdateServiceRequestValidation.validateListUpdatesOutput(response))
            throw new ValidationError('Response validation failed!', UpdateServiceRequestValidation.validateListUpdatesOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Create new update information
     *
     * @param updateInformation
     * The update to be created.
     * @param options.url
     * Url of the Update Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * JSON representation of the new update
     *
     * @category Update Service Methods
     */
    async createUpdate(updateInformation, options) {
        const url = appendToUrl(options?.url ?? this._url, '/updates');
        const body = updateInformation;
        if (!UpdateServiceRequestValidation.validateCreateUpdateInput(body))
            throw new ValidationError('Request validation failed!', UpdateServiceRequestValidation.validateCreateUpdateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!UpdateServiceRequestValidation.validateCreateUpdateOutput(response))
            throw new ValidationError('Response validation failed!', UpdateServiceRequestValidation.validateCreateUpdateOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Get update for device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param options.current_version
     * Current firmware version of the device
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 200: Device is already up-to-date
     * 303: New version is available
     *
     * @category Update Service Methods
     */
    async getUpdate(url, options) {
        const urlSuffix = '/updates/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/updates/{}');
        const parameters = {
            device_id: device_id,
            current_version: options?.current_version,
        };
        const query = [];
        if (parameters['current_version'])
            query.push([
                'current_version',
                parameters['current_version'].toString(),
            ]);
        if (!UpdateServiceRequestValidation.validateGetUpdateInput(parameters))
            throw new ValidationError('Request validation failed!', UpdateServiceRequestValidation.validateGetUpdateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: 'GET',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!UpdateServiceRequestValidation.validateGetUpdateOutput(response))
            throw new ValidationError('Response validation failed!', UpdateServiceRequestValidation.validateGetUpdateOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
    /**
     * Edit update information
     *
     * @param url
     * Url of the resource to be accessed.
     * @param updateInformation
     * Information regarding an update. Contains the id of the device,
     * the name of the latest version and a link to it.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Update information edited successfully
     *
     * @category Update Service Methods
     */
    async patchUpdate(url, updateInformation) {
        const urlSuffix = '/updates/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/updates/{}');
        const body = updateInformation;
        const parameters = {
            device_id: device_id,
        };
        if (!UpdateServiceRequestValidation.validatePatchUpdateInput(parameters, body))
            throw new ValidationError('Request validation failed!', UpdateServiceRequestValidation.validatePatchUpdateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'PATCH',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
            body: JSON.stringify(body),
        });
        if (!UpdateServiceRequestValidation.validatePatchUpdateOutput(response))
            throw new ValidationError('Response validation failed!', UpdateServiceRequestValidation.validatePatchUpdateOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
    /**
     * Delete update information
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Update information was deleted successfully
     *
     * @category Update Service Methods
     */
    async deleteUpdate(url) {
        const urlSuffix = '/updates/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id] = validateUrl(url, '/updates/{}');
        const parameters = {
            device_id: device_id,
        };
        if (!UpdateServiceRequestValidation.validateDeleteUpdateInput(parameters))
            throw new ValidationError('Request validation failed!', UpdateServiceRequestValidation.validateDeleteUpdateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: 'DELETE',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', authorization],
            ],
        });
        if (!UpdateServiceRequestValidation.validateDeleteUpdateOutput(response))
            throw new ValidationError('Response validation failed!', UpdateServiceRequestValidation.validateDeleteUpdateOutput.errors);
        if (isErrorResponse(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    }
}
//# sourceMappingURL=client.js.map