import { AuthenticationServiceSignatures, BookingServiceSignatures, DeviceServiceSignatures, ExperimentServiceSignatures, FederationServiceSignatures, UpdateServiceSignatures } from './signatures';
import { ResponseData } from './types';
/**
 * Namespace containing the request validation functions for the Authentication Service.
 */
export declare namespace AuthenticationServiceRequestValidation {
    /**
     * This function validates the inputs to login()
     */
    function validateLoginInput(body: AuthenticationServiceSignatures.LoginBody): boolean;
    /**
     * This function validates the outputs of login()
     */
    function validateLoginOutput(response: ResponseData): response is AuthenticationServiceSignatures.LoginResponse;
    /**
     * This function validates the inputs to logout()
     */
    function validateLogoutInput(body: AuthenticationServiceSignatures.LogoutBody): boolean;
    /**
     * This function validates the outputs of logout()
     */
    function validateLogoutOutput(response: ResponseData): response is AuthenticationServiceSignatures.LogoutResponse;
    /**
     * This function validates the inputs to createDeviceAuthenticationToken()
     */
    function validateCreateDeviceAuthenticationTokenInput(parameters: AuthenticationServiceSignatures.CreateDeviceAuthenticationTokenParameters): boolean;
    /**
     * This function validates the outputs of createDeviceAuthenticationToken()
     */
    function validateCreateDeviceAuthenticationTokenOutput(response: ResponseData): response is AuthenticationServiceSignatures.CreateDeviceAuthenticationTokenResponse;
    /**
     * This function validates the inputs to listUsers()
     */
    function validateListUsersInput(): boolean;
    /**
     * This function validates the outputs of listUsers()
     */
    function validateListUsersOutput(response: ResponseData): response is AuthenticationServiceSignatures.ListUsersResponse;
    /**
     * This function validates the inputs to createUser()
     */
    function validateCreateUserInput(body: AuthenticationServiceSignatures.CreateUserBody): boolean;
    /**
     * This function validates the outputs of createUser()
     */
    function validateCreateUserOutput(response: ResponseData): response is AuthenticationServiceSignatures.CreateUserResponse;
    /**
     * This function validates the inputs to getUser()
     */
    function validateGetUserInput(parameters: AuthenticationServiceSignatures.GetUserParameters): boolean;
    /**
     * This function validates the outputs of getUser()
     */
    function validateGetUserOutput(response: ResponseData): response is AuthenticationServiceSignatures.GetUserResponse;
    /**
     * This function validates the inputs to updateUser()
     */
    function validateUpdateUserInput(parameters: AuthenticationServiceSignatures.UpdateUserParameters, body: AuthenticationServiceSignatures.UpdateUserBody): boolean;
    /**
     * This function validates the outputs of updateUser()
     */
    function validateUpdateUserOutput(response: ResponseData): response is AuthenticationServiceSignatures.UpdateUserResponse;
    /**
     * This function validates the inputs to deleteUser()
     */
    function validateDeleteUserInput(parameters: AuthenticationServiceSignatures.DeleteUserParameters): boolean;
    /**
     * This function validates the outputs of deleteUser()
     */
    function validateDeleteUserOutput(response: ResponseData): response is AuthenticationServiceSignatures.DeleteUserResponse;
    /**
     * This function validates the inputs to getRolesOfUser()
     */
    function validateGetRolesOfUserInput(parameters: AuthenticationServiceSignatures.GetRolesOfUserParameters): boolean;
    /**
     * This function validates the outputs of getRolesOfUser()
     */
    function validateGetRolesOfUserOutput(response: ResponseData): response is AuthenticationServiceSignatures.GetRolesOfUserResponse;
    /**
     * This function validates the inputs to addRolesToUser()
     */
    function validateAddRolesToUserInput(parameters: AuthenticationServiceSignatures.AddRolesToUserParameters, body: AuthenticationServiceSignatures.AddRolesToUserBody): boolean;
    /**
     * This function validates the outputs of addRolesToUser()
     */
    function validateAddRolesToUserOutput(response: ResponseData): response is AuthenticationServiceSignatures.AddRolesToUserResponse;
    /**
     * This function validates the inputs to removeRolesFromUser()
     */
    function validateRemoveRolesFromUserInput(parameters: AuthenticationServiceSignatures.RemoveRolesFromUserParameters, body: AuthenticationServiceSignatures.RemoveRolesFromUserBody): boolean;
    /**
     * This function validates the outputs of removeRolesFromUser()
     */
    function validateRemoveRolesFromUserOutput(response: ResponseData): response is AuthenticationServiceSignatures.RemoveRolesFromUserResponse;
    /**
     * This function validates the inputs to listRoles()
     */
    function validateListRolesInput(): boolean;
    /**
     * This function validates the outputs of listRoles()
     */
    function validateListRolesOutput(response: ResponseData): response is AuthenticationServiceSignatures.ListRolesResponse;
    /**
     * This function validates the inputs to createRole()
     */
    function validateCreateRoleInput(body: AuthenticationServiceSignatures.CreateRoleBody): boolean;
    /**
     * This function validates the outputs of createRole()
     */
    function validateCreateRoleOutput(response: ResponseData): response is AuthenticationServiceSignatures.CreateRoleResponse;
    /**
     * This function validates the inputs to getRole()
     */
    function validateGetRoleInput(parameters: AuthenticationServiceSignatures.GetRoleParameters): boolean;
    /**
     * This function validates the outputs of getRole()
     */
    function validateGetRoleOutput(response: ResponseData): response is AuthenticationServiceSignatures.GetRoleResponse;
    /**
     * This function validates the inputs to updateRole()
     */
    function validateUpdateRoleInput(parameters: AuthenticationServiceSignatures.UpdateRoleParameters, body: AuthenticationServiceSignatures.UpdateRoleBody): boolean;
    /**
     * This function validates the outputs of updateRole()
     */
    function validateUpdateRoleOutput(response: ResponseData): response is AuthenticationServiceSignatures.UpdateRoleResponse;
    /**
     * This function validates the inputs to deleteRole()
     */
    function validateDeleteRoleInput(parameters: AuthenticationServiceSignatures.DeleteRoleParameters): boolean;
    /**
     * This function validates the outputs of deleteRole()
     */
    function validateDeleteRoleOutput(response: ResponseData): response is AuthenticationServiceSignatures.DeleteRoleResponse;
    /**
     * This function validates the inputs to getUsersWithRole()
     */
    function validateGetUsersWithRoleInput(parameters: AuthenticationServiceSignatures.GetUsersWithRoleParameters): boolean;
    /**
     * This function validates the outputs of getUsersWithRole()
     */
    function validateGetUsersWithRoleOutput(response: ResponseData): response is AuthenticationServiceSignatures.GetUsersWithRoleResponse;
    /**
     * This function validates the inputs to addUsersToRole()
     */
    function validateAddUsersToRoleInput(parameters: AuthenticationServiceSignatures.AddUsersToRoleParameters, body: AuthenticationServiceSignatures.AddUsersToRoleBody): boolean;
    /**
     * This function validates the outputs of addUsersToRole()
     */
    function validateAddUsersToRoleOutput(response: ResponseData): response is AuthenticationServiceSignatures.AddUsersToRoleResponse;
    /**
     * This function validates the inputs to removeUsersFromRole()
     */
    function validateRemoveUsersFromRoleInput(parameters: AuthenticationServiceSignatures.RemoveUsersFromRoleParameters, body: AuthenticationServiceSignatures.RemoveUsersFromRoleBody): boolean;
    /**
     * This function validates the outputs of removeUsersFromRole()
     */
    function validateRemoveUsersFromRoleOutput(response: ResponseData): response is AuthenticationServiceSignatures.RemoveUsersFromRoleResponse;
    /**
     * This function validates the inputs to getIdentity()
     */
    function validateGetIdentityInput(): boolean;
    /**
     * This function validates the outputs of getIdentity()
     */
    function validateGetIdentityOutput(response: ResponseData): response is AuthenticationServiceSignatures.GetIdentityResponse;
    /**
     * This function validates the inputs to updateIdentity()
     */
    function validateUpdateIdentityInput(body: AuthenticationServiceSignatures.UpdateIdentityBody): boolean;
    /**
     * This function validates the outputs of updateIdentity()
     */
    function validateUpdateIdentityOutput(response: ResponseData): response is AuthenticationServiceSignatures.UpdateIdentityResponse;
    /**
     * This function validates the inputs to register()
     */
    function validateRegisterInput(body: AuthenticationServiceSignatures.RegisterBody): boolean;
    /**
     * This function validates the outputs of register()
     */
    function validateRegisterOutput(response: ResponseData): response is AuthenticationServiceSignatures.RegisterResponse;
}
/**
 * Namespace containing the request validation functions for the Booking Service.
 */
export declare namespace BookingServiceRequestValidation {
    /**
     * This function validates the inputs to getSchedule()
     */
    function validateGetScheduleInput(body: BookingServiceSignatures.GetScheduleBody): boolean;
    /**
     * This function validates the outputs of getSchedule()
     */
    function validateGetScheduleOutput(response: ResponseData): response is BookingServiceSignatures.GetScheduleResponse;
    /**
     * This function validates the inputs to bookExperiment()
     */
    function validateBookExperimentInput(body: BookingServiceSignatures.BookExperimentBody): boolean;
    /**
     * This function validates the outputs of bookExperiment()
     */
    function validateBookExperimentOutput(response: ResponseData): response is BookingServiceSignatures.BookExperimentResponse;
    /**
     * This function validates the inputs to updateBooking()
     */
    function validateUpdateBookingInput(parameters: BookingServiceSignatures.UpdateBookingParameters, body: BookingServiceSignatures.UpdateBookingBody): boolean;
    /**
     * This function validates the outputs of updateBooking()
     */
    function validateUpdateBookingOutput(response: ResponseData): response is BookingServiceSignatures.UpdateBookingResponse;
    /**
     * This function validates the inputs to cancelBooking()
     */
    function validateCancelBookingInput(parameters: BookingServiceSignatures.CancelBookingParameters): boolean;
    /**
     * This function validates the outputs of cancelBooking()
     */
    function validateCancelBookingOutput(response: ResponseData): response is BookingServiceSignatures.CancelBookingResponse;
    /**
     * This function validates the inputs to getBooking()
     */
    function validateGetBookingInput(parameters: BookingServiceSignatures.GetBookingParameters): boolean;
    /**
     * This function validates the outputs of getBooking()
     */
    function validateGetBookingOutput(response: ResponseData): response is BookingServiceSignatures.GetBookingResponse;
    /**
     * This function validates the inputs to deleteBooking()
     */
    function validateDeleteBookingInput(parameters: BookingServiceSignatures.DeleteBookingParameters): boolean;
    /**
     * This function validates the outputs of deleteBooking()
     */
    function validateDeleteBookingOutput(response: ResponseData): response is BookingServiceSignatures.DeleteBookingResponse;
    /**
     * This function validates the inputs to lockBooking()
     */
    function validateLockBookingInput(parameters: BookingServiceSignatures.LockBookingParameters): boolean;
    /**
     * This function validates the outputs of lockBooking()
     */
    function validateLockBookingOutput(response: ResponseData): response is BookingServiceSignatures.LockBookingResponse;
    /**
     * This function validates the inputs to unlockBooking()
     */
    function validateUnlockBookingInput(parameters: BookingServiceSignatures.UnlockBookingParameters): boolean;
    /**
     * This function validates the outputs of unlockBooking()
     */
    function validateUnlockBookingOutput(response: ResponseData): response is BookingServiceSignatures.UnlockBookingResponse;
}
/**
 * Namespace containing the request validation functions for the Device Service.
 */
export declare namespace DeviceServiceRequestValidation {
    /**
     * This function validates the inputs to listDevices()
     */
    function validateListDevicesInput(): boolean;
    /**
     * This function validates the outputs of listDevices()
     */
    function validateListDevicesOutput(response: ResponseData): response is DeviceServiceSignatures.ListDevicesResponse;
    /**
     * This function validates the inputs to createDevice()
     */
    function validateCreateDeviceInput(parameters: DeviceServiceSignatures.CreateDeviceParameters, body: DeviceServiceSignatures.CreateDeviceBody): boolean;
    /**
     * This function validates the outputs of createDevice()
     */
    function validateCreateDeviceOutput(response: ResponseData): response is DeviceServiceSignatures.CreateDeviceResponse;
    /**
     * This function validates the inputs to getDevice()
     */
    function validateGetDeviceInput(parameters: DeviceServiceSignatures.GetDeviceParameters): boolean;
    /**
     * This function validates the outputs of getDevice()
     */
    function validateGetDeviceOutput(response: ResponseData): response is DeviceServiceSignatures.GetDeviceResponse;
    /**
     * This function validates the inputs to updateDevice()
     */
    function validateUpdateDeviceInput(parameters: DeviceServiceSignatures.UpdateDeviceParameters, body: DeviceServiceSignatures.UpdateDeviceBody): boolean;
    /**
     * This function validates the outputs of updateDevice()
     */
    function validateUpdateDeviceOutput(response: ResponseData): response is DeviceServiceSignatures.UpdateDeviceResponse;
    /**
     * This function validates the inputs to deleteDevice()
     */
    function validateDeleteDeviceInput(parameters: DeviceServiceSignatures.DeleteDeviceParameters): boolean;
    /**
     * This function validates the outputs of deleteDevice()
     */
    function validateDeleteDeviceOutput(response: ResponseData): response is DeviceServiceSignatures.DeleteDeviceResponse;
    /**
     * This function validates the inputs to instantiateDevice()
     */
    function validateInstantiateDeviceInput(parameters: DeviceServiceSignatures.InstantiateDeviceParameters): boolean;
    /**
     * This function validates the outputs of instantiateDevice()
     */
    function validateInstantiateDeviceOutput(response: ResponseData): response is DeviceServiceSignatures.InstantiateDeviceResponse;
    /**
     * This function validates the inputs to addAvailabilityRules()
     */
    function validateAddAvailabilityRulesInput(parameters: DeviceServiceSignatures.AddAvailabilityRulesParameters, body: DeviceServiceSignatures.AddAvailabilityRulesBody): boolean;
    /**
     * This function validates the outputs of addAvailabilityRules()
     */
    function validateAddAvailabilityRulesOutput(response: ResponseData): response is DeviceServiceSignatures.AddAvailabilityRulesResponse;
    /**
     * This function validates the inputs to createWebsocketToken()
     */
    function validateCreateWebsocketTokenInput(parameters: DeviceServiceSignatures.CreateWebsocketTokenParameters): boolean;
    /**
     * This function validates the outputs of createWebsocketToken()
     */
    function validateCreateWebsocketTokenOutput(response: ResponseData): response is DeviceServiceSignatures.CreateWebsocketTokenResponse;
    /**
     * This function validates the inputs to sendSignalingMessage()
     */
    function validateSendSignalingMessageInput(parameters: DeviceServiceSignatures.SendSignalingMessageParameters, body: DeviceServiceSignatures.SendSignalingMessageBody): boolean;
    /**
     * This function validates the outputs of sendSignalingMessage()
     */
    function validateSendSignalingMessageOutput(response: ResponseData): response is DeviceServiceSignatures.SendSignalingMessageResponse;
    /**
     * This function validates the inputs to listPeerconnections()
     */
    function validateListPeerconnectionsInput(): boolean;
    /**
     * This function validates the outputs of listPeerconnections()
     */
    function validateListPeerconnectionsOutput(response: ResponseData): response is DeviceServiceSignatures.ListPeerconnectionsResponse;
    /**
     * This function validates the inputs to createPeerconnection()
     */
    function validateCreatePeerconnectionInput(parameters: DeviceServiceSignatures.CreatePeerconnectionParameters, body: DeviceServiceSignatures.CreatePeerconnectionBody): boolean;
    /**
     * This function validates the outputs of createPeerconnection()
     */
    function validateCreatePeerconnectionOutput(response: ResponseData): response is DeviceServiceSignatures.CreatePeerconnectionResponse;
    /**
     * This function validates the inputs to getPeerconnection()
     */
    function validateGetPeerconnectionInput(parameters: DeviceServiceSignatures.GetPeerconnectionParameters): boolean;
    /**
     * This function validates the outputs of getPeerconnection()
     */
    function validateGetPeerconnectionOutput(response: ResponseData): response is DeviceServiceSignatures.GetPeerconnectionResponse;
    /**
     * This function validates the inputs to deletePeerconnection()
     */
    function validateDeletePeerconnectionInput(parameters: DeviceServiceSignatures.DeletePeerconnectionParameters): boolean;
    /**
     * This function validates the outputs of deletePeerconnection()
     */
    function validateDeletePeerconnectionOutput(response: ResponseData): response is DeviceServiceSignatures.DeletePeerconnectionResponse;
    /**
     * This function validates the inputs to patchPeerconnectionDeviceStatus()
     */
    function validatePatchPeerconnectionDeviceStatusInput(parameters: DeviceServiceSignatures.PatchPeerconnectionDeviceStatusParameters, body: DeviceServiceSignatures.PatchPeerconnectionDeviceStatusBody): boolean;
    /**
     * This function validates the outputs of patchPeerconnectionDeviceStatus()
     */
    function validatePatchPeerconnectionDeviceStatusOutput(response: ResponseData): response is DeviceServiceSignatures.PatchPeerconnectionDeviceStatusResponse;
}
/**
 * Namespace containing the request validation functions for the Experiment Service.
 */
export declare namespace ExperimentServiceRequestValidation {
    /**
     * This function validates the inputs to listExperiments()
     */
    function validateListExperimentsInput(): boolean;
    /**
     * This function validates the outputs of listExperiments()
     */
    function validateListExperimentsOutput(response: ResponseData): response is ExperimentServiceSignatures.ListExperimentsResponse;
    /**
     * This function validates the inputs to createExperiment()
     */
    function validateCreateExperimentInput(body: ExperimentServiceSignatures.CreateExperimentBody): boolean;
    /**
     * This function validates the outputs of createExperiment()
     */
    function validateCreateExperimentOutput(response: ResponseData): response is ExperimentServiceSignatures.CreateExperimentResponse;
    /**
     * This function validates the inputs to getExperiment()
     */
    function validateGetExperimentInput(parameters: ExperimentServiceSignatures.GetExperimentParameters): boolean;
    /**
     * This function validates the outputs of getExperiment()
     */
    function validateGetExperimentOutput(response: ResponseData): response is ExperimentServiceSignatures.GetExperimentResponse;
    /**
     * This function validates the inputs to updateExperiment()
     */
    function validateUpdateExperimentInput(parameters: ExperimentServiceSignatures.UpdateExperimentParameters, body: ExperimentServiceSignatures.UpdateExperimentBody): boolean;
    /**
     * This function validates the outputs of updateExperiment()
     */
    function validateUpdateExperimentOutput(response: ResponseData): response is ExperimentServiceSignatures.UpdateExperimentResponse;
    /**
     * This function validates the inputs to deleteExperiment()
     */
    function validateDeleteExperimentInput(parameters: ExperimentServiceSignatures.DeleteExperimentParameters): boolean;
    /**
     * This function validates the outputs of deleteExperiment()
     */
    function validateDeleteExperimentOutput(response: ResponseData): response is ExperimentServiceSignatures.DeleteExperimentResponse;
}
/**
 * Namespace containing the request validation functions for the Federation Service.
 */
export declare namespace FederationServiceRequestValidation {
    /**
     * This function validates the inputs to listInstitutions()
     */
    function validateListInstitutionsInput(): boolean;
    /**
     * This function validates the outputs of listInstitutions()
     */
    function validateListInstitutionsOutput(response: ResponseData): response is FederationServiceSignatures.ListInstitutionsResponse;
    /**
     * This function validates the inputs to createInstitution()
     */
    function validateCreateInstitutionInput(body: FederationServiceSignatures.CreateInstitutionBody): boolean;
    /**
     * This function validates the outputs of createInstitution()
     */
    function validateCreateInstitutionOutput(response: ResponseData): response is FederationServiceSignatures.CreateInstitutionResponse;
    /**
     * This function validates the inputs to getInstitution()
     */
    function validateGetInstitutionInput(parameters: FederationServiceSignatures.GetInstitutionParameters): boolean;
    /**
     * This function validates the outputs of getInstitution()
     */
    function validateGetInstitutionOutput(response: ResponseData): response is FederationServiceSignatures.GetInstitutionResponse;
    /**
     * This function validates the inputs to updateInstitution()
     */
    function validateUpdateInstitutionInput(parameters: FederationServiceSignatures.UpdateInstitutionParameters, body: FederationServiceSignatures.UpdateInstitutionBody): boolean;
    /**
     * This function validates the outputs of updateInstitution()
     */
    function validateUpdateInstitutionOutput(response: ResponseData): response is FederationServiceSignatures.UpdateInstitutionResponse;
    /**
     * This function validates the inputs to deleteInstitution()
     */
    function validateDeleteInstitutionInput(parameters: FederationServiceSignatures.DeleteInstitutionParameters): boolean;
    /**
     * This function validates the outputs of deleteInstitution()
     */
    function validateDeleteInstitutionOutput(response: ResponseData): response is FederationServiceSignatures.DeleteInstitutionResponse;
}
/**
 * Namespace containing the request validation functions for the Update Service.
 */
export declare namespace UpdateServiceRequestValidation {
    /**
     * This function validates the inputs to listUpdates()
     */
    function validateListUpdatesInput(): boolean;
    /**
     * This function validates the outputs of listUpdates()
     */
    function validateListUpdatesOutput(response: ResponseData): response is UpdateServiceSignatures.ListUpdatesResponse;
    /**
     * This function validates the inputs to createUpdate()
     */
    function validateCreateUpdateInput(body: UpdateServiceSignatures.CreateUpdateBody): boolean;
    /**
     * This function validates the outputs of createUpdate()
     */
    function validateCreateUpdateOutput(response: ResponseData): response is UpdateServiceSignatures.CreateUpdateResponse;
    /**
     * This function validates the inputs to getUpdate()
     */
    function validateGetUpdateInput(parameters: UpdateServiceSignatures.GetUpdateParameters): boolean;
    /**
     * This function validates the outputs of getUpdate()
     */
    function validateGetUpdateOutput(response: ResponseData): response is UpdateServiceSignatures.GetUpdateResponse;
    /**
     * This function validates the inputs to patchUpdate()
     */
    function validatePatchUpdateInput(parameters: UpdateServiceSignatures.PatchUpdateParameters, body: UpdateServiceSignatures.PatchUpdateBody): boolean;
    /**
     * This function validates the outputs of patchUpdate()
     */
    function validatePatchUpdateOutput(response: ResponseData): response is UpdateServiceSignatures.PatchUpdateResponse;
    /**
     * This function validates the inputs to deleteUpdate()
     */
    function validateDeleteUpdateInput(parameters: UpdateServiceSignatures.DeleteUpdateParameters): boolean;
    /**
     * This function validates the outputs of deleteUpdate()
     */
    function validateDeleteUpdateOutput(response: ResponseData): response is UpdateServiceSignatures.DeleteUpdateResponse;
}
//# sourceMappingURL=requestValidation.d.ts.map